@@ -5,8 +5,10 @@ import re
 from copy import deepcopy
 import numpy.matlib as mat
 import sys
+import os
+from ..updaters.tools import dicwrap
 # This used to import sympycore, but should now also work with sympy
-import sympy as SP
+import sympycore as SP
 
 
 #TODO: why have these as nested functions?
@@ -147,11 +149,15 @@ from __future__ import division
                 counter = counter + 1
             elif '...' not in x:
                 if counter == 0:
+                    # Just in case people have used [1] markers...
+                    if ']' in x: x = x.split(']')[1].lstrip().rstrip()
                     list_tmp1.append(x)
                 elif counter > 0:
                     str_tmp = ''
                     for y in mansys[i1-counter:i1+1]:
                         str_tmp = str_tmp + y.replace('...','')
+                    # Just in case people have used [1] markers...
+                    if ']' in str_tmp: str_tmp = str_tmp.split(']')[1].lstrip().rstrip()
                     list_tmp1.append(str_tmp)
                     counter = 0 
             i1=i1+1
@@ -165,12 +171,15 @@ from __future__ import division
         indx = []
         ssidic={}
         list_tmp = []
+        anyssi_found = False
         i1=0
         counter=0
         for x in secs['manualss'][0]:
             if mreg.search(x):
+                anyssi_found = True
                 ma = mreg.search(x)
-                str1 = ma.group().split('=')[0].split(']')[1].strip()
+                if ']' in ma.group(): str1 = ma.group().split('=')[0].split(']')[1].strip()
+                else: str1 = ma.group().split('=')[0].lstrip().rstrip()
                 str2 = ma.group().split('=')[1].strip()
                 ssidic[str1] = eval(str2)
                 indx.append(i1)
@@ -186,7 +195,11 @@ from __future__ import division
                     list_tmp.append(str_tmp.split(']')[1].split('=')[0].replace(';','').strip())
                     counter = 0 
             i1=i1+1
-        self.ssidic = ssidic
+        if anyssi_found:
+            self.ssidic = ssidic
+        else:
+            #Make empty to be filled by other procedure
+            self.ssidic = {}
         self.ssys_list = list_tmp
 
     return self
@@ -728,6 +741,8 @@ def mk_subs_dic(self, secs):
         list_tmp2.append([splitline[0].strip(), splitline[1].strip()])
 
     self.nlsubs_raw1 = deepcopy(list_tmp2)
+    self.nlsubsdic = dict(list_tmp2)
+    
     return self
 
 def subs_in_subs(self):
@@ -886,6 +901,19 @@ def mk_steady(self):
     self.nlsubs_raw2 = deepcopy(list_tmp2)  
     return self
 
+def mk_steady2(self):
+    list_tmp2 = deepcopy(self.foceqs2)
+    patup = ('{-10,10}|None','endo|con|exo|other','{-10,10}')
+    vreg = self.vreg
+    for i1,elem in enumerate(list_tmp2):
+        varli = list(vreg(patup,list_tmp2[i1],True,'max'))
+        varli.reverse()
+        for varo in varli:
+            vpos = varo[3]
+            list_tmp2[i1] = list_tmp2[i1][:vpos[0]]+varo[2][1]+'_bar'+list_tmp2[i1][vpos[1]:]
+    self.foceqss = deepcopy(list_tmp2)  
+    return self
+
 def mk_timeshift(self):
     list_tmp2 = deepcopy(self.nlsubs_raw2)
     _mreg = '(?<!DI)FF[_](?P<fint>\d{1,2})\{.*?\}'
@@ -1205,7 +1233,12 @@ def differ_out(self):
             for elem in self.ssidic.keys():
                 tmp_dic[elem] = SP.Symbol(elem)
             locals().update(tmp_dic)
-            expr = eval(evalstr)
+            try:
+                expr = eval(evalstr)
+            except:
+                print "Parse ERROR: Could not evaluate/differentiate expression: "
+                print list_tmp2[kk1][0]+" = "+list_tmp2[kk1][1]
+                sys.exit()
             resstr = expr.diff(locals()[str(differo_new)])
             resstr = str(resstr)
             # Now switch back to normal notation with brackets
@@ -1243,10 +1276,10 @@ def differ_out(self):
     self.nlsubs_list = deepcopy(list_tmp2)
     return self
 
-def mknonlinsys(self, secs):
-    """
-    Create Non-Linear FOC System
-    """
+def premknonlinsys(self,secs):
+    '''
+    This takes the raw lines of the FOCs and does some joining, splitting and stripping
+    '''
     # Make the non-linear system by joining lines and stripping
     variables = deepcopy(self.subs_vars)
     list_tmp1 = []
@@ -1257,20 +1290,29 @@ def mknonlinsys(self, secs):
             linecounter += 1
         else:
             if linecounter == 0:
-                line = x.replace(';','').split(']')[1].strip()
+                if ']' in x: line = x.replace(';','').split(']')[1].strip()
+                else: line = x.replace(';','').strip()
                 line = line.split('=')[0].strip()
                 list_tmp1.append(line)
             elif linecounter > 0:
                 str_tmp = ''
                 for y in secs['focs'][0][i1-linecounter:i1+1]:
-                    str_tmp += y.replace('...','').replace('\\',
-                                                           '').replace(';','').strip()
-                line = str_tmp.split(']')[1].strip()
+                    if ']' in y: str_tmp += y.split(']')[1].replace('...','').replace('\\','').replace(';','').strip()
+                    else: str_tmp += y.replace('...','').replace('\\','').replace(';','').strip()
+                if ']' in x: line = str_tmp.split(']')[1].strip()
+                else: line = str_tmp.strip()
                 linecounter = 0 
                 line = line.split('=')[0].strip()
                 list_tmp1.append(line)
         i1 += 1
+    self.foceqs = deepcopy(list_tmp1)
+    return self
 
+def premknonlinsys2(self, secs):
+    list_tmp1 = deepcopy(self.foceqs)
+    variables = deepcopy(self.subs_vars)
+    i1 = 0    
+    for x in list_tmp1:
         # substitute out in main nonlinear equation system
         list_tmp2 = deepcopy(self.nlsubs_list)
         mreg = re.compile('@(E\(t.*?\)\|){0,1}.*?\(t.*?\)')
@@ -1284,6 +1326,17 @@ def mknonlinsys(self, secs):
                 rhs_eq = rhs_eq[:pos]+'('+list_tmp2[indx][1]+')'+\
                     rhs_eq[poe:]
             list_tmp1[i] = rhs_eq
+        i1+=1
+    self.foceqs2 = deepcopy(list_tmp1)
+    return self
+
+def mknonlinsys(self, secs):
+    """
+    Create Non-Linear FOC System
+    """
+    list_tmp1 = deepcopy(self.foceqs2)
+    variables = deepcopy(self.subs_vars)
+    list_tmp2 = deepcopy(self.nlsubs_list)
 
     self, list_tmp1 = mkaug2(self, list_tmp1)
     list_tmp3 = [x[1] for x in list_tmp2]
@@ -1576,13 +1629,35 @@ def mk_msstate_subs(self):
     self.ssys_list = deepcopy(tmp_list)
     return self
 
+#This function is needed in population stage 1, at the end
+def mk_cfstate_subs(self):
+    """
+    This function takes the closed form defined numerical sstate conditions and then replaces
+    any @terms with the corresponding substitutions
+    """
+    _mreg = '@[a-zA-Z]*_bar'
+    mreg = re.compile(_mreg)
+    tmp_list = deepcopy(self.manss_sys)
+    sub_dic = deepcopy(self.nlsubs)
+    for i1,x in enumerate(tmp_list):
+        while mreg.search(tmp_list[i1]):
+            ma = mreg.search(tmp_list[i1])
+            str_tmp = ma.group()
+            tmp_list[i1] = tmp_list[i1].replace(str_tmp,'('+sub_dic[str_tmp]+')')
+    self.manss_sys = deepcopy(tmp_list)
+    return self
+
+# Extra population stage factored out, which is needed before steady state calculations
+def populate_model_stage_one_a(self, secs):
+    # Check and calculate the substitution list and dictionary
+    if any([False if 'None' in x else True for x in secs['vsfocs'][0]]):
+        # Create the raw nlsubs list 1
+        self = mk_subs_dic(self, secs)
 
 # Extra population stage factored out, which is needed before steady state calculations
 def populate_model_stage_one_b(self, secs):
     # Check and calculate the substitution list and dictionary
     if any([False if 'None' in x else True for x in secs['vsfocs'][0]]):
-        # Create the raw nlsubs list 1
-        self = mk_subs_dic(self, secs)
         # Create the raw nlsubs list 2 by replacing substitutions inside substitutions
         self = subs_in_subs(self)
         # Apply steady state transformation where needed
@@ -1595,11 +1670,26 @@ def populate_model_stage_one_b(self, secs):
         self = mk_steady(self)
         # Make second differentiation pass for remaining DIFFs
         self = differ_out(self)
+    return self
+
+def populate_model_stage_one_bb(self, secs):
     # Do substitutions inside the numerical steady state list
     # Check and do substitutions
     if any([False if 'None' in x else True for x in secs['vsfocs'][0]]) and\
        any([False if 'None' in x else True for x in secs['manualss'][0]]):
         self = mk_msstate_subs(self)
+    # Do substitutions inside the closed form steady state list
+    # Check and do substitutions
+    if any([False if 'None' in x else True for x in secs['vsfocs'][0]]) and\
+       any([False if 'None' in x else True for x in secs['closedformss'][0]]):
+        self = mk_cfstate_subs(self)
+    # Prepare the nonlinear FOC system but only stripping the raw format
+    if any([False if 'None' in x else True for x in secs['focs'][0]]):
+        self = premknonlinsys(self,secs)
+        # This takes the stripped system and does @ replacements
+        self = premknonlinsys2(self,secs)
+        # Also create a steady state version
+        self = mk_steady2(self)
     return self
 
 
