@@ -3,9 +3,10 @@
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
     config = Configuration('dsge', parent_package, top_path)
-    config.add_data_dir('tests')
     config.add_subpackage('solvers')
     config.add_subpackage('parsers')
+    config.add_subpackage('updaters')
+    config.add_subpackage('updaters_queued')
     return config
 
 if __name__ == '__main__':
